const sdk = {
	l10n:        require("sdk/l10n"),
	simpleprefs: require("sdk/simple-prefs"),
	ui:          require("sdk/ui"),
	
	system: {
		unload: require("sdk/system/unload")
	},
	util: {
		object: require("sdk/util/object")
	}
};

const tools = {
	menupopup:     require("../tools/menupopup"),
	searchengines: require("../tools/search-engines"),
	
	ui: {
		button: {
			highlight: require("../tools/ui/button/highlight")
		}
	}
};

const m = {
	ui: {
		manager: require("../ui/manager")
	}
};

const _ = sdk.l10n.get;



const XMLNS_XUL = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";



function Handler()
{
	sdk.system.unload.when(this.uninit.bind(this));
}
Handler.prototype = {
	buildEngineMenu: function()
	{
		function add_engines(menu, engines, callback)
		{
			for(var engine of engines) {
				var menuitem = new tools.menupopup.Item({
					title: engine.name,
					icon:  (engine.iconURI ? engine.iconURI.spec : null),
					
					onclick: (function(engine)
					{
						return function(event)
						{
							callback(engine, event);
						};
					})(engine)
				});
				
				// Highlight default engine
				if(engine == tools.searchengines.currentEngine) {
					menuitem.highlighted = true;
				}
				
				menu.content.push(menuitem);
			}
		}
		
		// Create menu
		var menu = new tools.menupopup.Menu();
		
		// Add all currently visible search engines to list
		add_engines(menu, tools.searchengines.getVisibleEngines(), function(engine, event)
		{
			tools.searchengines.currentEngine = engine;
		});
		
		// Add search engines offered by the current webpage to list
		var engines_offered = tools.searchengines.getOfferedEngines()
		if(engines_offered.length > 0) {
			// Add separator
			menu.content.push(new tools.menupopup.Separator());
			
			// Add correct engine text
			engines_offered = engines_offered.map(function(engine)
			{
				return sdk.util.object.extend({}, engine, {
					name: _("Add \"%s\"", engine.name)
				});
			});
			
			// Add list of engines offered by website
			add_engines(menu, engines_offered, function(engine, event)
			{
				tools.searchengines.addEngine(
						engine.uri,
						tools.searchengines.CONST.DATA_XML,
						(engine.iconURI ? engine.iconURI.spec : null),
						sdk.simpleprefs.prefs["display-addengine-dialog"]
				);
			});
		}
		
		// Add the final separator
		menu.content.push(new tools.menupopup.Separator());
		
		// Main actor
		menu.content.push(new tools.menupopup.Item({
			"title": _("Manage search engines..."),
			
			onclick: function(event)
			{
				m.ui.manager.open();
			}
		}));
		
		return menu;
	},
	
	
	openEngineList: function()
	{
		// Build search engine menu
		var menu = this.buildEngineMenu();
		
		// Untoggle toolbar button on close
		menu.once("close", function(menu)
		{
			this.button.state("window", {
				checked: false
			});
		}.bind(this));
		
		// Display menu
		menu.show({
			position: this.button
		});
	},
	
	
	init: function()
	{
		if(typeof(this.button) !== "object") {
			// Show the toolbar button
			this.button = sdk.ui.ToggleButton({
				id:    "manage-search-engines",
				label: _("Manage search engines"),
				icon: {
					"16": "./icon-16.png",
					"32": "./icon-32.png",
					"64": "./icon-64.png"
				},
	
				onChange: function(state)
				{
					if(state.checked) {
						// Open the list of search engines as a menupopup
						// (not a shitty SDK panel...)
						this.openEngineList();
					}
				}.bind(this)
			});
		}
	},
	
	setHighlight: function(window, highlighted)
	{
		// Apply highlighting
		if(highlighted) {
			tools.ui.button.highlight.enable(this.button, [window]);
		} else {
			tools.ui.button.highlight.disable(this.button, [window]);
		}
	},
	
	uninit: function()
	{
		if(typeof(this.button) === "object") {
			this.button.destroy();
			delete this.button;
		}
	}
};

module.exports = {
	Handler: Handler
};
