const sdk = {
	l10n:        require("sdk/l10n"),
	panel:       require("sdk/panel"),
	simpleprefs: require("sdk/simple-prefs"),
	system:      require("sdk/system")
};

const _ = sdk.l10n.get;



function show_compatibility_error(version_min)
{
	var software = sdk.system.name + " " + version_min;
	sdk.panel.Panel({
		contentURL: "data:text/html," + _("This feature requires at least %s", software),
		
		width:  600,
		height: 35
	}).show();
}



var handler;
function reload_handler(pref_name)
{
	// Stop previous handler
	if(typeof(handler) === "object") {
		handler.uninit();
	}
	
	// Load the new handler module
	var handler_module;
	if(parseInt(sdk.system.version.split(".")[0]) >= 30) {
		// Both of these implementations require APIs that are only available in Firefox 30+
		if(sdk.simpleprefs.prefs["display-menupopup"]) {
			handler_module = require("./handlers/menu");
		} else {
			handler_module = require("./handlers/popup");
		}
	} else {
		// Disable preference to show current state
		sdk.simpleprefs.prefs["display-menupopup"] = false;
		
		// Show warning if the change was triggered by the user
		if(pref_name) {
			show_compatibility_error(30);
		}
		
		handler_module = require("./handlers/popup_widget");
	}
	
	// Initialize new handler
	handler = new handler_module.Handler();
	handler.init();
}

// Load handler the first time
reload_handler(false);

// Reload on pref change
sdk.simpleprefs.on("display-menupopup", reload_handler);



// Setup button highlighter
if(parseInt(sdk.system.version.split(".")[0]) >= 30) {
	var EngineMonitor = require("./monitor").EngineMonitor;
	
	// Create highlighting monitor
	var monitor = new EngineMonitor(function(window)
	{
		handler.setHighlight(window, true);
	}, function(window)
	{
		handler.setHighlight(window, false);
	});
	
	function toggle_monitor()
	{
		// Start or stop monitoring based on pref
		if(sdk.simpleprefs.prefs["display-addengine-notification"]) {
			monitor.init();
		} else {
			monitor.uninit();
		}
	}

	// Start highlighter if it is supposed to run
	toggle_monitor();

	// Change on pref change
	sdk.simpleprefs.on("display-addengine-notification", toggle_monitor);
} else {
	// Show that the feature is not enabled
	sdk.simpleprefs.prefs["display-addengine-notification"] = false;
	
	// Provide feedback if the user tries to enable it anyways
	sdk.simpleprefs.on("display-addengine-notification", function()
	{
		sdk.simpleprefs.prefs["display-addengine-notification"] = false;
		
		show_compatibility_error(30);
	});
}
